@@ -18,11 +18,12 @@ To enable simple substitutions, add the match to find_substitutions.
 """
 from __future__ import print_function, division
 
-from collections import namedtuple
+from collections import namedtuple, defaultdict
 
 import sympy
 
-from sympy.core.compatibility import reduce
+from sympy.core.compatibility import reduce, Mapping, iterable
+from sympy.core.containers import Dict
 from sympy.core.logic import fuzzy_not
 from sympy.functions.elementary.trigonometric import TrigonometricFunction
 from sympy.functions.special.polynomials import OrthogonalPolynomial
@@ -30,6 +31,8 @@ from sympy.functions.elementary.piecewise import Piecewise
 from sympy.strategies.core import switch, do_one, null_safe, condition
 from sympy.core.relational import Eq, Ne
 from sympy.polys.polytools import degree
+from sympy.ntheory.factor_ import divisors
+from sympy.utilities.misc import debug
 
 ZERO = sympy.S.Zero
 
@@ -74,6 +77,19 @@ LegendreRule = Rule("LegendreRule", "n")
 HermiteRule = Rule("HermiteRule", "n")
 LaguerreRule = Rule("LaguerreRule", "n")
 AssocLaguerreRule = Rule("AssocLaguerreRule", "n a")
+CiRule = Rule("CiRule", "a b")
+ChiRule = Rule("ChiRule", "a b")
+EiRule = Rule("EiRule", "a b")
+SiRule = Rule("SiRule", "a b")
+ShiRule = Rule("ShiRule", "a b")
+ErfRule = Rule("ErfRule", "a b c")
+FresnelCRule = Rule("FresnelCRule", "a b c")
+FresnelSRule = Rule("FresnelSRule", "a b c")
+LiRule = Rule("LiRule", "a b")
+PolylogRule = Rule("PolylogRule", "a b")
+UpperGammaRule = Rule("UpperGammaRule", "a e")
+EllipticFRule = Rule("EllipticFRule", "a d")
+EllipticERule = Rule("EllipticERule", "a d")
 
 IntegralInfo = namedtuple('IntegralInfo', 'integrand symbol')
 
@@ -123,6 +139,36 @@ def manual_diff(f, symbol):
                 return f.args[0] * manual_diff(f.args[1], symbol)
     return f.diff(symbol)
 
+def manual_subs(expr, *args):
+    """
+    A wrapper for `expr.subs(*args)` with additional logic for substitution
+    of invertible functions.
+    """
+    if len(args) == 1:
+        sequence = args[0]
+        if isinstance(sequence, (Dict, Mapping)):
+            sequence = sequence.items()
+        elif not iterable(sequence):
+            raise ValueError("Expected an iterable of (old, new) pairs")
+    elif len(args) == 2:
+        sequence = [args]
+    else:
+        raise ValueError("subs accepts either 1 or 2 arguments")
+
+    new_subs = []
+    for old, new in sequence:
+        if isinstance(old, sympy.log):
+            # If log(x) = y, then exp(a*log(x)) = exp(a*y)
+            # that is, x**a = exp(a*y). Replace nontrivial powers of x
+            # before subs turns them into `exp(y)**a`, but
+            # do not replace x itself yet, to avoid `log(exp(y))`.
+            x0 = old.args[0]
+            expr = expr.replace(lambda x: x.is_Pow and x.base == x0,
+                lambda x: sympy.exp(x.exp*new))
+            new_subs.append((x0, sympy.exp(new)))
+
+    return expr.subs(list(sequence) + new_subs)
+
 # Method based on that on SIN, described in "Symbolic Integration: The
 # Stormy Decade"
 
@@ -130,13 +176,22 @@ def find_substitutions(integrand, symbol, u_var):
     results = []
 
     def test_subterm(u, u_diff):
+        if u_diff == 0:
+            return False
         substituted = integrand / u_diff
         if symbol not in substituted.free_symbols:
             # replaced everything already
             return False
+        debug("substituted: {}, u: {}, u_var: {}".format(substituted, u, u_var))
+        substituted = manual_subs(substituted, u, u_var).cancel()
 
-        substituted = substituted.subs(u, u_var).cancel()
         if symbol not in substituted.free_symbols:
+            # avoid increasing the degree of a rational function
+            if integrand.is_rational_function(symbol) and substituted.is_rational_function(u_var):
+                deg_before = max([degree(t, symbol) for t in integrand.as_numer_denom()])
+                deg_after = max([degree(t, u_var) for t in substituted.as_numer_denom()])
+                if deg_after > deg_before:
+                    return False
             return substituted.as_independent(u_var, as_Add=False)
 
         # special treatment for substitutions u = (a*x+b)**(1/n)
@@ -173,10 +228,18 @@ def find_substitutions(integrand, symbol, u_var):
                 r.extend(possible_subterms(u))
             return r
         elif isinstance(term, sympy.Pow):
+            r = []
             if term.args[1].is_constant(symbol):
-                return [term.args[0]]
+                r.append(term.args[0])
             elif term.args[0].is_constant(symbol):
-                return [term.args[1]]
+                r.append(term.args[1])
+            if term.args[1].is_Integer:
+                r.extend([term.args[0]**d for d in divisors(term.args[1])
+                    if 1 < d < abs(term.args[1])])
+                if term.args[0].is_Add:
+                    r.extend([t for t in possible_subterms(term.args[0])
+                        if t.is_Pow])
+            return r
         elif isinstance(term, sympy.Add):
             r = []
             for arg in term.args:
@@ -204,6 +267,7 @@ def rewriter(condition, rewrite):
     """Strategy that rewrites an integrand."""
     def _rewriter(integral):
         integrand, symbol = integral
+        debug("Integral: {} is rewritten with {} on symbol: {}".format(integrand, rewrite, symbol))
         if condition(*integral):
             rewritten = rewrite(*integral)
             if rewritten != integrand:
@@ -220,6 +284,7 @@ def proxy_rewriter(condition, rewrite):
     def _proxy_rewriter(criteria):
         criteria, integral = criteria
         integrand, symbol = integral
+        debug("Integral: {} is rewritten with {} on symbol: {} and criteria: {}".format(integrand, rewrite, symbol, criteria))
         args = criteria + list(integral)
         if condition(*args):
             rewritten = rewrite(*args)
@@ -242,7 +307,12 @@ def alternatives(*rules):
     """Strategy that makes an AlternativeRule out of multiple possible results."""
     def _alternatives(integral):
         alts = []
+        count = 0
+        debug("List of Alternative Rules")
         for rule in rules:
+            count = count + 1
+            debug("Rule {}: {}".format(count, rule))
+
             result = rule(integral)
             if (result and not isinstance(result, DontKnowRule) and
                 result != integral and result not in alts):
@@ -314,6 +384,46 @@ def orthogonal_poly_rule(integral):
                     return orthogonal_poly_classes[klass](*args)
 
 
+def special_function_rule(integral):
+    integrand, symbol = integral
+    a = sympy.Wild('a', exclude=[symbol], properties=[lambda x: not x.is_zero])
+    b = sympy.Wild('b', exclude=[symbol])
+    c = sympy.Wild('c', exclude=[symbol])
+    d = sympy.Wild('d', exclude=[symbol], properties=[lambda x: not x.is_zero])
+    e = sympy.Wild('e', exclude=[symbol], properties=[
+        lambda x: not (x.is_nonnegative and x.is_integer)])
+    wilds = (a, b, c, d, e)
+    # patterns consist of a SymPy class, a wildcard expr, an optional
+    # condition coded as a lambda (when Wild properties are not enough),
+    # followed by an applicable rule
+    patterns = (
+        (sympy.Mul, sympy.exp(a*symbol + b)/symbol, None, EiRule),
+        (sympy.Mul, sympy.cos(a*symbol + b)/symbol, None, CiRule),
+        (sympy.Mul, sympy.cosh(a*symbol + b)/symbol, None, ChiRule),
+        (sympy.Mul, sympy.sin(a*symbol + b)/symbol, None, SiRule),
+        (sympy.Mul, sympy.sinh(a*symbol + b)/symbol, None, ShiRule),
+        (sympy.Pow, 1/sympy.log(a*symbol + b), None, LiRule),
+        (sympy.exp, sympy.exp(a*symbol**2 + b*symbol + c), None, ErfRule),
+        (sympy.sin, sympy.sin(a*symbol**2 + b*symbol + c), None, FresnelSRule),
+        (sympy.cos, sympy.cos(a*symbol**2 + b*symbol + c), None, FresnelCRule),
+        (sympy.Mul, symbol**e*sympy.exp(a*symbol), None, UpperGammaRule),
+        (sympy.Mul, sympy.polylog(b, a*symbol)/symbol, None, PolylogRule),
+        (sympy.Pow, 1/sympy.sqrt(a - d*sympy.sin(symbol)**2),
+            lambda a, d: a != d, EllipticFRule),
+        (sympy.Pow, sympy.sqrt(a - d*sympy.sin(symbol)**2),
+            lambda a, d: a != d, EllipticERule),
+    )
+    for p in patterns:
+        if isinstance(integrand, p[0]):
+            match = integrand.match(p[1])
+            if match:
+                wild_vals = tuple(match.get(w) for w in wilds
+                                  if match.get(w) is not None)
+                if p[2] is None or p[2](*wild_vals):
+                    args = wild_vals + (integrand, symbol)
+                    return p[3](*args)
+
+
 def inverse_trig_rule(integral):
     integrand, symbol = integral
     base, exp = integrand.as_base_exp()
@@ -387,7 +497,6 @@ def add_rule(integral):
 
 def mul_rule(integral):
     integrand, symbol = integral
-    args = integrand.args
 
     # Constant times function case
     coeff, f = integrand.as_independent(symbol)
@@ -404,7 +513,9 @@ def _parts_rule(integrand, symbol):
     # log, inverse trig, algebraic, trigonometric, exponential
     def pull_out_algebraic(integrand):
         integrand = integrand.cancel().together()
-        algebraic = [arg for arg in integrand.args if arg.is_algebraic_expr(symbol)]
+        # iterating over Piecewise args would not work here
+        algebraic = ([] if isinstance(integrand, sympy.Piecewise)
+            else [arg for arg in integrand.args if arg.is_algebraic_expr(symbol)])
         if algebraic:
             u = sympy.Mul(*algebraic)
             dv = (integrand / u).cancel()
@@ -458,19 +569,32 @@ def _parts_rule(integrand, symbol):
             # Can integrate a polynomial times OrthogonalPolynomial
             if rule == pull_out_algebraic and isinstance(dv, OrthogonalPolynomial):
                     v_step = integral_steps(dv, symbol)
-                    if isinstance(v_step, DontKnowRule):
+                    if contains_dont_know(v_step):
                         return
                     else:
                         du = u.diff(symbol)
                         v = _manualintegrate(v_step)
                         return u, dv, v, du, v_step
 
-            for rule in liate_rules[index + 1:]:
-                r = rule(integrand)
-                # make sure dv is amenable to integration
-                if r and r[0].subs(dummy, 1).equals(dv):
-                    du = u.diff(symbol)
-                    v_step = integral_steps(sympy.simplify(dv), symbol)
+            # make sure dv is amenable to integration
+            accept = False
+            if index < 2:  # log and inverse trig are usually worth trying
+                accept = True
+            elif (rule == pull_out_algebraic and dv.args and
+                all(isinstance(a, (sympy.sin, sympy.cos, sympy.exp))
+                for a in dv.args)):
+                    accept = True
+            else:
+                for rule in liate_rules[index + 1:]:
+                    r = rule(integrand)
+                    if r and r[0].subs(dummy, 1).equals(dv):
+                        accept = True
+                        break
+
+            if accept:
+                du = u.diff(symbol)
+                v_step = integral_steps(sympy.simplify(dv), symbol)
+                if not contains_dont_know(v_step):
                     v = _manualintegrate(v_step)
                     return u, dv, v, du, v_step
 
@@ -484,16 +608,26 @@ def parts_rule(integral):
     steps = []
     if result:
         u, dv, v, du, v_step = result
+        debug("u : {}, dv : {}, v : {}, du : {}, v_step: {}".format(u, dv, v, du, v_step))
         steps.append(result)
 
         if isinstance(v, sympy.Integral):
             return
 
-        while True:
-            if (integrand / (v * du)).cancel() == 1:
+        # Set a limit on the number of times u can be used
+        if isinstance(u, (sympy.sin, sympy.cos, sympy.exp, sympy.sinh, sympy.cosh)):
+            cachekey = u.xreplace({symbol: _cache_dummy})
+            if _parts_u_cache[cachekey] > 2:
+                return
+            _parts_u_cache[cachekey] += 1
+
+        # Try cyclic integration by parts a few times
+        for _ in range(4):
+            debug("Cyclic integration {} with v: {}, du: {}, integrand: {}".format(_, v, du, integrand))
+            coefficient = ((v * du) / integrand).cancel()
+            if coefficient == 1:
                 break
-            if symbol not in (integrand / (v * du)).cancel().free_symbols:
-                coefficient = ((v * du) / integrand).cancel()
+            if symbol not in coefficient.free_symbols:
                 rule = CyclicPartsRule(
                     [PartsRule(u, dv, v_step, None, None, None)
                      for (u, dv, v, du, v_step) in steps],
@@ -505,11 +639,15 @@ def parts_rule(integral):
                                              constant * integrand, symbol)
                 return rule
 
-            result = _parts_rule(v * du, symbol)
+            # _parts_rule is sensitive to constants, factor it out
+            next_constant, next_integrand = (v * du).as_coeff_Mul()
+            result = _parts_rule(next_integrand, symbol)
 
             if result:
                 u, dv, v, du, v_step = result
-                steps.append(result)
+                u *= next_constant
+                du *= next_constant
+                steps.append((u, dv, v, du, v_step))
             else:
                 break
 
@@ -606,16 +744,64 @@ def quadratic_denom_rule(integral):
     a = sympy.Wild('a', exclude=[symbol])
     b = sympy.Wild('b', exclude=[symbol])
     c = sympy.Wild('c', exclude=[symbol])
+
     match = integrand.match(a / (b * symbol ** 2 + c))
 
-    if not match:
-        return
+    if match:
+        a, b, c = match[a], match[b], match[c]
+        if b.is_extended_real and c.is_extended_real:
+            return PiecewiseRule([(ArctanRule(a, b, c, integrand, symbol), sympy.Gt(c / b, 0)),
+                                (ArccothRule(a, b, c, integrand, symbol), sympy.And(sympy.Gt(symbol ** 2, -c / b), sympy.Lt(c / b, 0))),
+                                (ArctanhRule(a, b, c, integrand, symbol), sympy.And(sympy.Lt(symbol ** 2, -c / b), sympy.Lt(c / b, 0))),
+            ], integrand, symbol)
+        else:
+            return ArctanRule(a, b, c, integrand, symbol)
 
-    a, b, c = match[a], match[b], match[c]
-    return PiecewiseRule([(ArctanRule(a, b, c, integrand, symbol), sympy.Gt(c / b, 0)),
-                          (ArccothRule(a, b, c, integrand, symbol), sympy.And(sympy.Gt(symbol ** 2, -c / b), sympy.Lt(c / b, 0))),
-                          (ArctanhRule(a, b, c, integrand, symbol), sympy.And(sympy.Lt(symbol ** 2, -c / b), sympy.Lt(c / b, 0))),
-    ], integrand, symbol)
+    d = sympy.Wild('d', exclude=[symbol])
+    match2 = integrand.match(a / (b * symbol ** 2 + c * symbol + d))
+    if match2:
+        b, c =  match2[b], match2[c]
+        if b.is_zero:
+            return
+        u = sympy.Dummy('u')
+        u_func = symbol + c/(2*b)
+        integrand2 = integrand.subs(symbol, u - c / (2*b))
+        next_step = integral_steps(integrand2, u)
+        if next_step:
+            return URule(u, u_func, None, next_step, integrand2, symbol)
+        else:
+            return
+    e = sympy.Wild('e', exclude=[symbol])
+    match3 = integrand.match((a* symbol + b) / (c * symbol ** 2 + d * symbol + e))
+    if match3:
+        a, b, c, d, e = match3[a], match3[b], match3[c], match3[d], match3[e]
+        if c.is_zero:
+            return
+        denominator = c * symbol**2 + d * symbol + e
+        const =  a/(2*c)
+        numer1 =  (2*c*symbol+d)
+        numer2 = - const*d + b
+        u = sympy.Dummy('u')
+        step1 = URule(u,
+                      denominator,
+                      const,
+                      integral_steps(u**(-1), u),
+                      integrand,
+                      symbol)
+        if const != 1:
+            step1 = ConstantTimesRule(const,
+                                      numer1/denominator,
+                                      step1,
+                                      const*numer1/denominator,
+                                      symbol)
+        if numer2.is_zero:
+            return step1
+        step2 = integral_steps(numer2/denominator, symbol)
+        substeps = AddRule([step1, step2], integrand, symbol)
+        rewriten = const*numer1/denominator+numer2/denominator
+        return RewriteRule(rewriten, substeps, integrand, symbol)
+
+    return
 
 def root_mul_rule(integral):
     integrand, symbol = integral
@@ -867,7 +1053,7 @@ def trig_substitution_rule(integral):
                 substitutions[sympy.sqrt(f(theta)**(-2))] = 1/f(theta)
 
             replaced = integrand.subs(symbol, x_func).trigsimp()
-            replaced = replaced.subs(substitutions)
+            replaced = manual_subs(replaced, substitutions)
             if not replaced.has(symbol):
                 replaced *= manual_diff(x_func, theta)
                 replaced = replaced.trigsimp()
@@ -899,10 +1085,15 @@ def substitution_rule(integral):
 
     u_var = sympy.Dummy("u")
     substitutions = find_substitutions(integrand, symbol, u_var)
+    count = 0
     if substitutions:
+        debug("List of Substitution Rules")
         ways = []
         for u_func, c, substituted in substitutions:
             subrule = integral_steps(substituted, u_var)
+            count = count + 1
+            debug("Rule {}: {}".format(count, subrule))
+
             if contains_dont_know(subrule):
                 continue
 
@@ -922,7 +1113,7 @@ def substitution_rule(integral):
 
                     for expr in could_be_zero:
                         if not fuzzy_not(expr.is_zero):
-                            substep = integral_steps(integrand.subs(expr, 0), symbol)
+                            substep = integral_steps(manual_subs(integrand, expr, 0), symbol)
 
                             if substep:
                                 piecewise.append((
@@ -999,8 +1190,13 @@ def rewrites_rule(integral):
 def fallback_rule(integral):
     return DontKnowRule(*integral)
 
-# Cache is used to break cyclic integrals
+# Cache is used to break cyclic integrals.
+# Need to use the same dummy variable in cached expressions for them to match.
+# Also record "u" of integration by parts, to avoid infinite repetition.
 _integral_cache = {}
+_parts_u_cache = defaultdict(int)
+_cache_dummy = sympy.Dummy("z")
+
 def integral_steps(integrand, symbol, **options):
     """Returns the steps needed to compute an integral.
 
@@ -1047,13 +1243,16 @@ def integral_steps(integrand, symbol, **options):
         to obtain a result.
 
     """
-    cachekey = (integrand, symbol)
+    cachekey = integrand.xreplace({symbol: _cache_dummy})
     if cachekey in _integral_cache:
         if _integral_cache[cachekey] is None:
-            # cyclic integral! null_safe will eliminate that path
-            return None
+            # Stop this attempt, because it leads around in a loop
+            return DontKnowRule(integrand, symbol)
         else:
-            return _integral_cache[cachekey]
+            # TODO: This is for future development, as currently
+            # _integral_cache gets no values other than None
+            return (_integral_cache[cachekey].xreplace(_cache_dummy, symbol),
+                symbol)
     else:
         _integral_cache[cachekey] = None
 
@@ -1083,6 +1282,7 @@ def integral_steps(integrand, symbol, **options):
         return _integral_is_subclass
 
     result = do_one(
+        null_safe(special_function_rule),
         null_safe(switch(key, {
             sympy.Pow: do_one(null_safe(power_rule), null_safe(inverse_trig_rule), \
                               null_safe(quadratic_denom_rule)),
@@ -1150,6 +1350,9 @@ def eval_add(substeps, integrand, symbol):
 @evaluates(URule)
 def eval_u(u_var, u_func, constant, substep, integrand, symbol):
     result = _manualintegrate(substep)
+    if u_func.is_Pow and u_func.exp == -1:
+        # avoid needless -log(1/x) from substitution
+        result = result.subs(sympy.log(u_var), -sympy.log(u_func.base))
     return result.subs(u_var, u_func)
 
 @evaluates(PartsRule)
@@ -1319,6 +1522,70 @@ def eval_laguerre(n, integrand, symbol):
 def eval_assoclaguerre(n, a, integrand, symbol):
     return -sympy.assoc_laguerre(n + 1, a - 1, symbol)
 
+@evaluates(CiRule)
+def eval_ci(a, b, integrand, symbol):
+    return sympy.cos(b)*sympy.Ci(a*symbol) - sympy.sin(b)*sympy.Si(a*symbol)
+
+@evaluates(ChiRule)
+def eval_chi(a, b, integrand, symbol):
+    return sympy.cosh(b)*sympy.Chi(a*symbol) + sympy.sinh(b)*sympy.Shi(a*symbol)
+
+@evaluates(EiRule)
+def eval_ei(a, b, integrand, symbol):
+    return sympy.exp(b)*sympy.Ei(a*symbol)
+
+@evaluates(SiRule)
+def eval_si(a, b, integrand, symbol):
+    return sympy.sin(b)*sympy.Ci(a*symbol) + sympy.cos(b)*sympy.Si(a*symbol)
+
+@evaluates(ShiRule)
+def eval_shi(a, b, integrand, symbol):
+    return sympy.sinh(b)*sympy.Chi(a*symbol) + sympy.cosh(b)*sympy.Shi(a*symbol)
+
+@evaluates(ErfRule)
+def eval_erf(a, b, c, integrand, symbol):
+    if a.is_extended_real:
+        return Piecewise(
+            (sympy.sqrt(sympy.pi/(-a))/2 * sympy.exp(c - b**2/(4*a)) *
+                sympy.erf((-2*a*symbol - b)/(2*sympy.sqrt(-a))), a < 0),
+            (sympy.sqrt(sympy.pi/a)/2 * sympy.exp(c - b**2/(4*a)) *
+                sympy.erfi((2*a*symbol + b)/(2*sympy.sqrt(a))), True))
+    else:
+        return sympy.sqrt(sympy.pi/a)/2 * sympy.exp(c - b**2/(4*a)) * \
+                sympy.erfi((2*a*symbol + b)/(2*sympy.sqrt(a)))
+
+@evaluates(FresnelCRule)
+def eval_fresnelc(a, b, c, integrand, symbol):
+    return sympy.sqrt(sympy.pi/(2*a)) * (
+        sympy.cos(b**2/(4*a) - c)*sympy.fresnelc((2*a*symbol + b)/sympy.sqrt(2*a*sympy.pi)) +
+        sympy.sin(b**2/(4*a) - c)*sympy.fresnels((2*a*symbol + b)/sympy.sqrt(2*a*sympy.pi)))
+
+@evaluates(FresnelSRule)
+def eval_fresnels(a, b, c, integrand, symbol):
+    return sympy.sqrt(sympy.pi/(2*a)) * (
+        sympy.cos(b**2/(4*a) - c)*sympy.fresnels((2*a*symbol + b)/sympy.sqrt(2*a*sympy.pi)) -
+        sympy.sin(b**2/(4*a) - c)*sympy.fresnelc((2*a*symbol + b)/sympy.sqrt(2*a*sympy.pi)))
+
+@evaluates(LiRule)
+def eval_li(a, b, integrand, symbol):
+    return sympy.li(a*symbol + b)/a
+
+@evaluates(PolylogRule)
+def eval_polylog(a, b, integrand, symbol):
+    return sympy.polylog(b + 1, a*symbol)
+
+@evaluates(UpperGammaRule)
+def eval_uppergamma(a, e, integrand, symbol):
+    return symbol**e * (-a*symbol)**(-e) * sympy.uppergamma(e + 1, -a*symbol)/a
+
+@evaluates(EllipticFRule)
+def eval_elliptic_f(a, d, integrand, symbol):
+    return sympy.elliptic_f(symbol, d/a)/sympy.sqrt(a)
+
+@evaluates(EllipticERule)
+def eval_elliptic_e(a, d, integrand, symbol):
+    return sympy.elliptic_e(symbol, d/a)*sympy.sqrt(a)
+
 @evaluates(DontKnowRule)
 def eval_dontknowrule(integrand, symbol):
     return sympy.Integral(integrand, symbol)
@@ -1336,7 +1603,7 @@ def manualintegrate(f, var):
     Compute indefinite integral of a single variable using an algorithm that
     resembles what a student would do by hand.
 
-    Unlike ``integrate``, var can only be a single symbol.
+    Unlike :func:`~.integrate`, var can only be a single symbol.
 
     Examples
     ========
@@ -1377,6 +1644,8 @@ def manualintegrate(f, var):
     sympy.integrals.integrals.Integral
     """
     result = _manualintegrate(integral_steps(f, var))
+    # Clear the cache of u-parts
+    _parts_u_cache.clear()
     # If we got Piecewise with two parts, put generic first
     if isinstance(result, Piecewise) and len(result.args) == 2:
         cond = result.args[0][1]
