@@ -5,7 +5,7 @@ the expression E**(pi*I) will be converted into -1
 the expression (x+x)**2 will be converted into 4*x**2
 """
 from .simplify import (simplify, hypersimp, hypersimilar,
-    logcombine, separatevars, posify, besselsimp,
+    logcombine, separatevars, posify, besselsimp, kroneckersimp,
     signsimp, bottom_up, nsimplify)
 
 from .fu import FU, fu
